from sys import argv

from PyQt4.QtGui import QDialog, QApplication
from PyQt4.QtCore import Qt

from twisted.internet.defer import inlineCallbacks
from autobahn.twisted.wamp import Application

from orexplore.lamp.lamp_controller_dialog_ui import Ui_LampControllerDialog

import qt4reactor

class LampControllerDialog(QDialog, Ui_LampControllerDialog):
    """ Dialog for controlling the lamp using WAMP RPC calls.
    """
    def __init__(self, app):
        """ Create a new LampControllerDialog.

        Args:
            app: The Autobahn ApplicationSession to use for RPC calls.
        """
        QDialog.__init__(self)
        self.setupUi(self)
        self.app = app

    @inlineCallbacks
    def on_descriptionEdit_textChanged(self, description):
        """ Handle changes in the description text edit.

        Args:
            description: The new description string.
        """
        yield self.app.session.call('com.orexplore.lamp.set_description',
                unicode(description.toUtf8().data(), encoding='UTF-8'))

    @inlineCallbacks
    def on_stateCheckBox_stateChanged(self, state):
        """ Handle changes in the state checkbox.

        Args:
            state: The new checkbox state (Qt.CheckState).
        """
        yield self.app.session.call('com.orexplore.lamp.set_state', state == Qt.Checked)
    
def main():
    qapp = QApplication(argv)
    app = Application('com.orexplore')

    dialog = LampControllerDialog(app)

    # Show the dialog upon joining the WAMP realm.
    @app.signal('onjoined')
    def on_joined():
        dialog.show()

    qt4reactor.install()

    from twisted.internet import reactor

    app.run(url = u"ws://localhost:8080/ws", realm = u"orexplore")
